capture log close
log using pvstata-test01, replace

//  pgm:        pvstata-test01.do
//  task:       test example set 1 of pvdata and pvstata
//  project:    pv stata
//  author:     jsl \ 2008-01-27; 2009-10-13; 2010-10-24

version 11.1
clear all
set scheme s2manual
local pgm pvstata-test01
local dte "2010-10-24"
local tag "`pgm'.do jsl `dte'"
local pngwid = 1400
run pvdata4.ado
run pvplot4.ado
run pvsel4.ado
run pvs.ado
run pvp.ado
which pvdata4
which pvsel4
which pvplot4

//  #1
//  Create pv dataset

use sgc-schiz-sdcat-usa01.dta, clear
gen notkids = sdchild>3
lab var notkids "Definitely not let person watch children"

pvdata4 sdneighb sdsocial sdchild sdfriend sdwork sdmarry, ///
   id(id_iu) othervars(notkids) auto(1)   ///
    method(stacked)

//  PART 1: basic operations

//  #2
//  Colors based on sdfriend

pvplot4 , ///
    colv(sdfriend)                      /// colors vary by this variable
    colors(red blue orange green)       ///
    gridspace(10)                       /// space from grid to paths
    gridcolor(black)                    /// color of grid lines
    gridxwidth(.4)                      /// width of x-grid lines
    gridywidth(.4)                      /// width of y-grid lines
    aspect(1)                           /// aspect ratio
/// graph options follow - these are passed directly to the twoway command
    title(#2: colors based on sdfriend, size(small)) ///
    caption(#2 `tag',size(tiny))      ///
    export(`pgm'-02.png, replace width(`pngwidth'))

//  #3
//  change colors of existing graph using interactive mode

pvplot4 , ///
    colors(pink blue orange green)      ///
    title(#3: interactive change of colors, size(small)) ///
    caption(#3 `tag',size(tiny))   ///
    export(`pgm'-03.png, replace width(`pngwidth'))

//  #4
//  turn off interactive mode by using reset
//  default values are used for all options

pvplot4 , intoff                        ///
    title(#4: plot using default values, size(small)) ///
    caption(#4 `tag',size(tiny))      ///
    export(`pgm'-04.png, replace width(`pngwidth'))

//  PART 2: use of colors

//  #5
//  only select colors

local plt = 5
pvplot4 , reset /// use default options
    colv(sdfriend) colors(red blue orange green) ///
    title("#`plt': colors only", size(small)) ///
    caption(#`plt' `tag',size(tiny)) ///
    export("`pgm'-0`plt'.png, replace width(`pngwidth')")

//  #6
//  colors and colintensity values

local plt = 6
pvplot4 , ///
    colv(sdfriend) colors(red blue orange green) ///
    colintensity(.2 .2 1 1) /// low intensity for first two categories
    title("#`plt': colors & colintensity", size(small)) ///
    caption(#`plt' `tag',size(tiny)) ///
    export("`pgm'-0`plt'.png, replace width(`pngwidth')")

//  #7
//  colors by first item; with intensity by sdwork item
//  low intensity for categories 1 and 2; high for 3 and 4

local plt = 7
pvplot4 , ///
    colv(sdfriend) colors(red blue orange green) ///
    int(.3 .3 1 1) intvar(sdwork) ///
    title("#`plt': color by sdfriend; intensity by sdwork", size(small)) ///
    caption(#`plt' `tag',size(tiny)) ///
    export("`pgm'-0`plt'.png, replace width(`pngwidth')")

//  #8
//  color by definitely not care for  kids

use sgc-schiz-sdcat-usa01.dta, clear
gen notkids = sdchild>3
lab var notkids "Definitely not let person watch children"

* do not select sdchild as item
pvdata4 sdneighb sdsocial sdfriend sdwork sdmarry, ///
   id(id_iu) othervars(notkids) auto(1)   ///
    method(stacked)

local plt = 8
pvplot4 , ///
    colv(notkids) colors(orange green) ///
    title("#`plt': color based on non-item variable", size(small)) ///
    caption(#`plt' `tag',size(tiny)) ///
    export("`pgm'-0`plt'.png, replace width(`pngwidth')")

//  #9
//  color by item 1 and intensity by notkids

local plt = 9
pvplot4 , ///
    colv(sdfriend) colors(red blue orange green) ///
    intv(notkids) intensity(.25 1) ///
    title("#`plt': color based on item 1; intensity by non-item variable", ///
        size(small)) ///
    caption(#`plt' `tag',size(tiny)) ///
    export("`pgm'-0`plt'.png, replace width(`pngwidth')")

//  #10
//  show only if notkids==1, otherwise shown with yellow with low intensity

local plt = 10
pvplot4 , ///
    showif(notkids==1) ///  show path colors if notkids
    hidecolor(yellow) hideint(.3) /// if notkids==0 use yellow low intensity
    colv(sdneighb) colors(red blue orange green) ///
    title("#`plt': show if notkids, else low intensity", size(small)) ///
    caption(#`plt' `tag',size(tiny)) ///
    export("`pgm'-`plt'.png, replace width(`pngwidth')")

//  #11
//  hide if notkids==1 and use low gray intensity

local plt = 11
pvplot4 , hideif(notkids==1) hidecolor(black) hideint(lo) ///
    colv(sdneighb) colors(red blue orange green) ///
    title("#`plt': show if notkids", size(small)) ///
    caption(#`plt' `tag',size(tiny)) ///
    export("`pgm'-`plt'.png, replace width(`pngwidth')") reset

log close
exit
